@@ -18,6 +18,8 @@ from nose.tools import assert_equal, assert_raises
 from botocore.client import ClientEndpointBridge
 from botocore.exceptions import NoRegionError
 
+from tests import BaseSessionTest, ClientHTTPStubber
+
 
 # NOTE: sqs endpoint updated to be the CN in the SSL cert because
 # a bug in python2.6 prevents subjectAltNames from being parsed
@@ -453,7 +455,8 @@ def test_known_endpoints():
     # fixed list of known endpoints.  This list doesn't need to be kept 100% up
     # to date, but serves as a basis for regressions as the endpoint data
     # logic evolves.
-    resolver = _get_patched_session().get_component('endpoint_resolver')
+    resolver = _get_patched_session()._get_internal_component(
+        'endpoint_resolver')
     for region_name, service_dict in KNOWN_REGIONS.items():
         for service_name, endpoint in service_dict.items():
             yield (_test_single_service_region, service_name,
@@ -472,7 +475,7 @@ def _test_single_service_region(service_name, region_name,
 def test_all_s3_endpoints_have_s3v4():
     session = _get_patched_session()
     partitions = session.get_available_partitions()
-    resolver = session.get_component('endpoint_resolver')
+    resolver = session._get_internal_component('endpoint_resolver')
     for partition_name in partitions:
         for endpoint in session.get_available_regions('s3', partition_name):
             resolved = resolver.construct_endpoint('s3', endpoint)
@@ -481,7 +484,8 @@ def test_all_s3_endpoints_have_s3v4():
 
 
 def test_known_endpoints():
-    resolver = _get_patched_session().get_component('endpoint_resolver')
+    resolver = _get_patched_session()._get_internal_component(
+        'endpoint_resolver')
     for service_name, endpoint in KNOWN_AWS_PARTITION_WIDE.items():
         yield (_test_single_service_partition_endpoint, service_name,
                endpoint, resolver)
@@ -495,5 +499,61 @@ def _test_single_service_partition_endpoint(service_name, expected_endpoint,
 
 
 def test_non_partition_endpoint_requires_region():
-    resolver = _get_patched_session().get_component('endpoint_resolver')
+    resolver = _get_patched_session()._get_internal_component(
+        'endpoint_resolver')
     assert_raises(NoRegionError, resolver.construct_endpoint, 'ec2')
+
+
+class TestEndpointResolution(BaseSessionTest):
+
+    def setUp(self):
+        super(TestEndpointResolution, self).setUp()
+        self.xml_response = (
+            b'<?xml version="1.0" encoding="UTF-8"?>\n\n'
+            b'<ListRolesResponse '
+            b'xmlns="https://iam.amazonaws.com/doc/2010-05-08/">\n'
+            b'<ListRolesResult>\n</ListRolesResult>'
+            b'</ListRolesResponse>'
+        )
+
+    def create_stubbed_client(self, service_name, region_name, **kwargs):
+        client = self.session.create_client(service_name, region_name, **kwargs)
+        http_stubber = ClientHTTPStubber(client)
+        http_stubber.start()
+        return client, http_stubber
+
+    def test_regionalized_client_endpoint_resolution(self):
+        client, stubber = self.create_stubbed_client('s3', 'us-east-2')
+        stubber.add_response()
+        client.list_buckets()
+        self.assertEqual(
+            stubber.requests[0].url,
+            'https://s3.us-east-2.amazonaws.com/'
+        )
+
+    def test_regionalized_client_with_unknown_region(self):
+        client, stubber = self.create_stubbed_client('s3', 'not-real')
+        stubber.add_response()
+        client.list_buckets()
+        # Validate we don't fall back to partition endpoint for
+        # regionalized services.
+        self.assertEqual(
+            stubber.requests[0].url,
+            'https://s3.not-real.amazonaws.com/'
+        )
+
+    def test_unregionalized_client_endpoint_resolution(self):
+        client, stubber = self.create_stubbed_client('iam', 'us-west-2')
+        stubber.add_response(body=self.xml_response)
+        client.list_roles()
+        self.assertTrue(
+            stubber.requests[0].url.startswith('https://iam.amazonaws.com/')
+        )
+
+    def test_unregionalized_client_with_unknown_region(self):
+        client, stubber = self.create_stubbed_client('iam', 'not-real')
+        stubber.add_response(body=self.xml_response)
+        client.list_roles()
+        self.assertTrue(
+            stubber.requests[0].url.startswith('https://iam.amazonaws.com/')
+        )
